import graphEntity from './graphEntityClient';

const { field, query, mutate, exclude } = graphEntity;

class TaskSchedule {
  @field('String') calendar;
  @field('Number') target;
  @field('String') start;
  @field('String') end;
  @field('Enum') beforeMeal;
  @field('Enum') mealType;
}

class AlertThreshold {
  @field('Enum') measure;
  @field('Number') ranges;
  @field('Enum') triggers;
  @field('String') unit;
}

class Task {
  @field('ID')             id;
  @field('Enum')         type;
  @field('String')         description;
  @field('Number')         chances;
  @field('Boolean')        activated;
  @field('Enum')         repeat;
  @field('String')         baselineWeight;
  @field('TaskSchedule')   schedule;
  @field('AlertThreshold') threshold;
  // @field('String') assessment: TaskAssessment
  // @field('String') notificationSettings: NotificationSettings
}

export default class AdminProgram {
  @field('ID')      id;
  @field('Date')    createdAt;
  @field('Date')    updatedAt;
  @field('User')    creator;

  // @field('String') organization: Organization

  @field('String')  name;
  @field('String')  description;
  @field('Enum')    status;
  @field('Enum')    healthCondition;
  @field('Task')    tasks;
  // @field('String') devices: [Device]
  @field({
    length: 'Number',
    calendar: 'Enum'
  })                duration;


  @query('adminProgramList', {
    data: 'AdminProgram',
    pageInfo: { total: 'Number', lastPage: 'Number' }
  })
  @exclude(['data.creator', 'data.tasks', 'data.duration'])
  static list() {
    return {
      page: 1,
      count: 100,
      options: {},
      sort: {}
    };
  }

  @query('adminProgram', 'AdminProgram')
  static get(id) {
    return { id };
  }

  @mutate('editAdminProgram', 'AdminProgram', {
    id: 'ID', name: 'String', description: 'String'
  })
  save() {
    return this;
  }
}
